using System;
using System.IO;
using System.Text;

namespace Steganografi
{
    /// <summary>
    /// Kelas ini merepresentasikan operasi penyimpanan informasi ke dalam cover.
    /// </summary>
	public class BMPCoverFile : ICoverFile
	{
		
		private System.Drawing.Bitmap Bmpfile;    // 

        public BMPCoverFile(System.Drawing.Bitmap fileBmp)
        {
			this.Bmpfile = fileBmp;
		}
		
		public IStegoFile CreateStegoFile(string stegoFileName, byte[] message,string name, string password,FileStatus status) {
			
			// Open the cover
			MemoryStream inStream = new MemoryStream();

            Bmpfile.Save(inStream,System.Drawing.Imaging.ImageFormat.Bmp);
            inStream.Position = 0;
			// Check whether the cover is a bmp file
			char b = (char) inStream.ReadByte();
			char m = (char) inStream.ReadByte();	
			if (!(b=='B' && m=='M'))
				throw new Exception("The file is not a bitmap! ");
				
			// Check whether the bitmap is 24bit
			byte[] buffer = new byte[2];
			inStream.Seek(28, 0);
			inStream.Read(buffer,0,2);			
			Int16 nBit = BitConverter.ToInt16(buffer, 0);						
			if (nBit!=24)
				throw new Exception("The file is not a 24bit bitmap!");
			
			// Read the header from the cover			
			int offset = 54;	// Header Size		
			byte[] header = new byte[offset];
			inStream.Seek(0, 0);
			inStream.Read(header, 0, offset);
			
			// Write the header to the stego file
			FileStream outStream = new FileStream(stegoFileName, FileMode.Create, FileAccess.Write);

			outStream.Write(header, 0, offset);
			
			// kompress the message if its needed
            byte[] cmessage = message;
            if (status == FileStatus.C)
                cmessage = CompressHelper.compressByte(message);
            // else , do nothing.

            UInt32 origSize = CompressHelper.OrigSize;
            if (origSize > message.Length)
            { }
            Console.WriteLine("Ukuran awal = {0} setelah dikompress = {1}",origSize, cmessage.Length);
			// Add the length of the message inside the first 4 bytes + panjang nama file yang tersimpan + 4 byte ukuran pesan.
            byte[] messageBytes = AddLengthAhead(cmessage, name, status, origSize);    
      
			// Encrypt the message
			messageBytes = CryptoHelper.Encrypt(messageBytes, password);
      
			// Encode the cover using the LSB method
     		inStream.Seek(offset, 0);			     	
     		LSBHelper.Encode(inStream, messageBytes, outStream);     	     	
     	     	
			inStream.Close();
			outStream.Close();
			
			// Return a new BMPStegoFile
			return new BMPStegoFile(stegoFileName, password);
			
		}
		
        /// <summary>
        /// Menambahkan Header, the header structure is
        /// [2b name length][nb name][4b message length][1byte status][4b 0 if its not compressed]
        /// </summary>
        /// <param name="messageBytes">array of byte of the message to be hide</param>
        /// <param name="name">name of the message file</param>
        /// <param name="status">state, compressed or not</param>
        /// <param name="origsize">the original size of the message file</param>
        /// <returns>header+message</returns>
		private byte[] AddLengthAhead(byte[] messageBytes,string name,FileStatus status,UInt32 origsize) {
			int len = messageBytes.Length;  // ukuran pesan.
            byte[] bname = UTF8Encoding.UTF8.GetBytes(name);
            int namelen = bname.Length;
            // periksa apakah length of the filename is valid.
            if (namelen > 99)
            {
                throw new Exception("The File name is too looong, max = 99!!");
            }
            string strnamelen = "";
            if (namelen < 10)
            {
                strnamelen = "0" + namelen.ToString();
            }
            else
            {
                strnamelen = namelen.ToString();
            }
            // akhir pemeriksaan.

            byte[] bnameLen = UTF8Encoding.UTF8.GetBytes(strnamelen);   // byte[] of the file size
			byte[] bLen = BitConverter.GetBytes(len);       // byte[] of the message size
            Console.WriteLine("ukuran nama ={0} nama = {1} bnamelen = {2} panjang pesan = {3} status = {4} origsize = {5}",
                UTF8Encoding.UTF8.GetString(bnameLen), UTF8Encoding.UTF8.GetString(bname), bnameLen.Length,
                BitConverter.ToInt32(bLen, 0),status.ToString(),origsize);
            byte[] bstatus = UTF8Encoding.UTF8.GetBytes(status.ToString());
            byte[] borigsize = BitConverter.GetBytes(origsize);
			byte[] ret = new byte[len + bLen.Length + bname.Length+ bnameLen.Length + bstatus.Length + borigsize.Length];
            

            for (int i = 0; i < bnameLen.Length; i++)       // panjang nama.
                ret[i] = bnameLen[i];
            for (int j = 0; j < bname.Length; j++)          // nama.
                ret[j + bnameLen.Length] = bname[j];
            for (int i = 0; i < bLen.Length; i ++ )         // panjang pesan.
                ret[i + bnameLen.Length + bname.Length] = bLen[i];
            for (int i = 0; i < bstatus.Length; i++)         // status.
                ret[i + bLen.Length + bname.Length + bnameLen.Length] = bstatus[i];
            for (int i = 0; i < borigsize.Length; i++)      // originalsize.
                ret[i + bLen.Length + bname.Length + bnameLen.Length + bstatus.Length] = borigsize[i];
            for (int i = 0; i < messageBytes.Length; i++)   // pesan.
                ret[i + bLen.Length + bname.Length + bnameLen.Length + bstatus.Length+borigsize.Length] = messageBytes[i];
			return ret;
		}
		
	}
	
}
