using System;
using System.IO;
using System.Text;

namespace Steganografi
{
	public class BMPStegoFile : IStegoFile
	{
		
		private string fileName;
		private string password;		
		private byte[] hiddenMessage;
        private string name;
        private int size;
				
		public BMPStegoFile(string fileName, string password) {
			this.fileName = fileName;
			this.password = password;
		}
		
		// Return the hidden message
		public byte[] HiddenMessage {
			get {
					if (hiddenMessage==null)
						ExtractHiddenMessage();						
					return hiddenMessage;
			}
		}
		
        // return the name of file message.
        public string nama
        {
            get {
                if (name == null)
                    ExtractHiddenMessage();
                return name;
            }
        }

        // return the size of the file
        public int Size
        {
            get {
                return size;
            }
        }

        /// <summary>
        /// Ekstraksi file (info) yang tersembunyi dalam stego file
        /// coro jowone : Extract the hidden file (message) from stego file
        /// </summary>
		protected void ExtractHiddenMessage() {
			
            /*Kamus : Dictionary, eh actually its glossary
             * byte[] bLen  : array of byte yang merepresentasikan nilai panjang nama file
             * int len      : bLen in integer type.
             * btye[] filename  : array of byte of the hidden filename.
             * byte[] lenNName  : bLen + filename.
             * string strstatus : status in string mode.
             * iOrigSize    : integer original size.
             */
			// Open the file
			Stream inStream = new FileStream(fileName, FileMode.Open, FileAccess.Read);		
			
			// Data starts at byte 54
			int offset = 54;						
			inStream.Seek(offset, 0);							
			
			// Decode the first 2 bytes which contain
			// the length of the message file name, using the LSB method
			byte[] bLen = LSBHelper.Decode(inStream, 2);					
			
			// Decript the length
			bLen = CryptoHelper.Decrypt(bLen, password);			
			int len = Convert.ToInt16(UTF8Encoding.UTF8.GetString(bLen));
            Console.WriteLine("instream = {0}\n len = {1}",inStream.Length, len);
            // maju 4 byte.
            inStream.Seek(offset + 2 * 8, 0);   // 54 + 16 =70 , panjang header dan ket ukuran nama.

            // decode the file name.
            //Console.WriteLine("posisi = " + inStream.Position);
            byte[] filename = LSBHelper.Decode(inStream,len);
            // concatenate first with previous bits in order not give wrong result
            // when its decrypted
            byte[] lenNName = ConcatByteArray(bLen, filename);

            // decrypt
            lenNName = CryptoHelper.Decrypt(lenNName,password);
            byte[] NName = GetByteMessage(lenNName,2);  // take only needed bytes

            name = UTF8Encoding.UTF8.GetString(NName); // save the nama
            Console.WriteLine("Nama File {0}",name);
			// Decode length of the message using the LSB method
			inStream.Seek(offset + 2*8 + len*8 , 0);
            byte[] bmLen = LSBHelper.Decode(inStream, 4);
            // concatenate first with previous bits in order not give wrong result
            // when its decrypted
            byte[] lenNameLen = ConcatByteArray(lenNName, bmLen);

            // Decript the length
            lenNameLen = CryptoHelper.Decrypt(lenNameLen, password);

            byte[] bmLen2 = GetByteMessage(lenNameLen,(2+len)); //take only needed bytes.
            
            int mlen = BitConverter.ToInt32(bmLen2, 0);
            size = mlen;
            Console.WriteLine("Panjang Pesan = {0}",mlen);
            
            inStream.Seek(offset + (2 * 8) + len * 8 + (4*8), 0);
            // decode status
            byte[] state = LSBHelper.Decode(inStream, 1);
            byte[] status = ConcatByteArray(lenNameLen,state);
            //decrypt.
            status = CryptoHelper.Decrypt(status,password);
            Console.WriteLine("Status = {0} len = {1}", status.ToString(), status.Length);
            // get status only.
            byte[] thestatus = GetByteMessage(status,(2+len+4));
            string strstatus = UTF8Encoding.UTF8.GetString(thestatus);  // status in string mode
            Console.WriteLine("Status = {0} len = {1}",strstatus,thestatus.Length);
            //inStream.Seek(offset + (2 * 8) + len * 8 + (4 * 8)+ (1*8), 0);
            inStream.Seek(offset + (7 + len) * 8 , 0);

            // decode original file size.
            byte[] origs = LSBHelper.Decode(inStream, 4);
            byte[] origsz = ConcatByteArray(status, origs);
            //decrypt.
            origsz = CryptoHelper.Decrypt(origsz, password);
            // get origsize only.
            byte[] origsize = GetByteMessage(origsz, (2 + len + 1 + 4));
            UInt32 iOrigsize = (uint)BitConverter.ToInt32(origsize,0);  // status in string mode
            Console.WriteLine("Status = {0} Original Size = {1} origsz = {2}",strstatus,iOrigsize, origsz.Length);
            //inStream.Seek(offset + (2 * 8) + len * 8 + (4 * 8)+ (1*8) + (4*8), 0);
            inStream.Seek(offset + (11 + len) * 8, 0);
            // decode the message using LSB method.

			byte[] buffer;

			try 
			{
                buffer = LSBHelper.Decode(inStream, mlen);									
			} catch {
				throw new Exception("Wrong password or not a stego file!");
			}
            // 2byte len of filename, len byte filename, 4byte message length, 1b status, 4b origsize
            // byte[] ourHeader = new byte[2+len+4+1+4]; 
            Console.WriteLine("Buffer Length 0 = {0} len = {1}", buffer.Length, (11+len));
            byte[] ourHeader = new byte[11+len];
            ourHeader = ConcatByteArray(bmLen,filename);    // file len + filename.
            ourHeader = ConcatByteArray(bLen,ourHeader);   // file len+ filename+messagesize .
            ourHeader = ConcatByteArray(ourHeader, state);   // file len+ filename+messagesize + status.
            ourHeader = ConcatByteArray(ourHeader, origs);   // file len+ filename+messagesize + status + original size.
            Console.WriteLine("header Length = {0} ", ourHeader.Length);
            // Add the length ahead (8*8+len bytes)
            buffer = ConcatByteArray(ourHeader, buffer);
            Console.WriteLine("Buffer Length 2= {0} ", buffer.Length);
			// Decript the message (including the first n bytes)
			buffer = CryptoHelper.Decrypt(buffer, password);
            Console.WriteLine("Buffer Length 3= {0} ", buffer.Length);
			// Extract only the message (without the first n bytes)
            // byte[] byteMessage = GetByteMessage(buffer,(2+len+4+1+4));
            byte[] byteMessage = GetByteMessage(buffer, (11+len));
            Console.WriteLine("Byte message = {0} len = {1}", byteMessage.Length, ourHeader.Length);
            // decompress. ?
            if (strstatus == "C")   // jika terkompresi
            {
                hiddenMessage = DecompressHelper.DecompressByte(byteMessage, iOrigsize);
            }
            else
            {
                hiddenMessage = byteMessage;
            }
			inStream.Close();

        }
        #region Next Project, Extract from non 24-bit Bitmap image
        /// <summary>
        /// Ekstraksi file (info) yang tersembunyi dalam stego file
        /// coro jowone : Extract the hidden file (message) from stego file
        /// </summary>
        /*protected void ExtractHiddenMessage2()
        {

            /*Kamus : Dictionary, eh actually its glossary
             * byte[] bLen  : array of byte yang merepresentasikan nilai panjang nama file
             * int len      : bLen in integer type.
             * btye[] filename  : array of byte of the hidden filename.
             * byte[] lenNName  : bLen + filename.
             * string strstatus : status in string mode.
             * iOrigSize    : integer original size.
             */
        /*/ Open the file
        MemoryStream inStream = new MemoryStream();
        if (fileName.Substring(fileName.LastIndexOf(".")) != ".bmp")
        //ubah ke file bitmap.
        {
            System.Drawing.Image img = System.Drawing.Image.FromFile(fileName);
            System.Drawing.Bitmap bmp = new System.Drawing.Bitmap(img.Width, img.Height, System.Drawing.Imaging.PixelFormat.Format24bppRgb);
            bmp.Save(ms, System.Drawing.Imaging.ImageFormat.Bmp);
        }
        inStream.Position = 0;
        //Stream inStream = new FileStream(fileName, FileMode.Open, FileAccess.Read);

        // Data starts at byte 54
        int offset = 54;
        inStream.Seek(offset, 0);

        // Decode the first 2 bytes which contain
        // the length of the message file name, using the LSB method
        byte[] bLen = LSBHelper.Decode(inStream, 2);

        // Decript the length
        bLen = CryptoHelper.Decrypt(bLen, password);
        int len = Convert.ToInt16(UTF8Encoding.UTF8.GetString(bLen));
        Console.WriteLine("instream = {0}\n len = {1}", inStream.Length, len);
        // maju 4 byte. why english so difficult 4 me :P, ganbatte
        inStream.Seek(offset + 2 * 8, 0);   // 54 + 16 =70 , panjang header dan ket ukuran nama.

        // decode the file name.
        //Console.WriteLine("posisi = " + inStream.Position);
        byte[] filename = LSBHelper.Decode(inStream, len);
        // concatenate first with previous bits in order not give wrong result
        // when its decrypted
        byte[] lenNName = ConcatByteArray(bLen, filename);

        // decrypt
        lenNName = CryptoHelper.Decrypt(lenNName, password);
        byte[] NName = GetByteMessage(lenNName, 2);  // take only needed bytes

        name = UTF8Encoding.UTF8.GetString(NName); // save the nama
        Console.WriteLine("Nama File {0}", name);
        // Decode length of the message using the LSB method
        inStream.Seek(offset + 2 * 8 + len * 8, 0);
        byte[] bmLen = LSBHelper.Decode(inStream, 4);
        // concatenate first with previous bits in order not give wrong result
        // when its decrypted
        byte[] lenNameLen = ConcatByteArray(lenNName, bmLen);

        // Decript the length
        lenNameLen = CryptoHelper.Decrypt(lenNameLen, password);

        byte[] bmLen2 = GetByteMessage(lenNameLen, (2 + len)); //take only needed bytes.

        int mlen = BitConverter.ToInt32(bmLen2, 0);
        size = mlen;
        Console.WriteLine("Panjang Pesan = {0}", mlen);

        inStream.Seek(offset + (2 * 8) + len * 8 + (4 * 8), 0);
        // decode status
        byte[] state = LSBHelper.Decode(inStream, 1);
        byte[] status = ConcatByteArray(lenNameLen, state);
        //decrypt.
        status = CryptoHelper.Decrypt(status, password);
        Console.WriteLine("Status = {0} len = {1}", status.ToString(), status.Length);
        // get status only.
        byte[] thestatus = GetByteMessage(status, (2 + len + 4));
        string strstatus = UTF8Encoding.UTF8.GetString(thestatus);  // status in string mode
        Console.WriteLine("Status = {0} len = {1}", strstatus, thestatus.Length);
        //inStream.Seek(offset + (2 * 8) + len * 8 + (4 * 8)+ (1*8), 0);
        inStream.Seek(offset + (7 + len) * 8, 0);

        // decode original file size.
        byte[] origs = LSBHelper.Decode(inStream, 4);
        byte[] origsz = ConcatByteArray(status, origs);
        //decrypt.
        origsz = CryptoHelper.Decrypt(origsz, password);
        // get origsize only.
        byte[] origsize = GetByteMessage(origsz, (2 + len + 1 + 4));
        UInt32 iOrigsize = (uint)BitConverter.ToInt32(origsize, 0);  // status in string mode
        Console.WriteLine("Status = {0} Original Size = {1} origsz = {2}", strstatus, iOrigsize, origsz.Length);
        //inStream.Seek(offset + (2 * 8) + len * 8 + (4 * 8)+ (1*8) + (4*8), 0);
        inStream.Seek(offset + (11 + len) * 8, 0);
        // decode the message using LSB method.

        byte[] buffer;

        try
        {
            buffer = LSBHelper.Decode(inStream, mlen);
        }
        catch
        {
            throw new Exception("Wrong password or not a stego file!");
        }
        // 2byte len of filename, len byte filename, 4byte message length, 1b status, 4b origsize
        // byte[] ourHeader = new byte[2+len+4+1+4]; 
        Console.WriteLine("Buffer Length 0 = {0} len = {1}", buffer.Length, (11 + len));
        byte[] ourHeader = new byte[11 + len];
        ourHeader = ConcatByteArray(bmLen, filename);    // file len + filename.
        ourHeader = ConcatByteArray(bLen, ourHeader);   // file len+ filename+messagesize .
        ourHeader = ConcatByteArray(ourHeader, state);   // file len+ filename+messagesize + status.
        ourHeader = ConcatByteArray(ourHeader, origs);   // file len+ filename+messagesize + status + original size.
        Console.WriteLine("header Length = {0} ", ourHeader.Length);
        // Add the length ahead (8*8+len bytes)
        buffer = ConcatByteArray(ourHeader, buffer);
        Console.WriteLine("Buffer Length 2= {0} ", buffer.Length);
        // Decript the message (including the first n bytes)
        buffer = CryptoHelper.Decrypt(buffer, password);
        Console.WriteLine("Buffer Length 3= {0} ", buffer.Length);
        // Extract only the message (without the first n bytes)
        // byte[] byteMessage = GetByteMessage(buffer,(2+len+4+1+4));
        byte[] byteMessage = GetByteMessage(buffer, (11 + len));
        Console.WriteLine("Byte message = {0} len = {1}", byteMessage.Length, ourHeader.Length);
        // decompress. ?
        if (strstatus == "C")   // jika terkompresi
        {
            hiddenMessage = DecompressHelper.DecompressByte(byteMessage, iOrigsize);
        }
        else
        {
            hiddenMessage = byteMessage;
        }
        inStream.Close();

    }*/
        #endregion

        // Concatenate two byte arrays
		private byte[] ConcatByteArray(byte[] a, byte[] b) {
			byte[] ret = new byte[a.Length + b.Length];
			for (int i=0; i < a.Length; i++)
				ret[i] = a[i];
			for (int i=0; i < b.Length; i++)
				ret[i+a.Length] = b[i];
			return ret;
		}
		
		// Return a the byte array in input without
		// the first 4 bytes which represent the
		// length of the message
		private byte[] GetByteMessage(byte[] buffer,int len) {
			byte[] ret = new byte[buffer.Length-len];
			for (int i=len; i < buffer.Length; i++)
				ret[i-len] = buffer[i];			
			return ret;
		}
	}	
}
