using System;
using System.IO;
using System.IO.Compression;

namespace Steganografi
{
    /// <summary>
    /// status apakah dikompresi = 1 atau tidak dikompresi = 0
    /// </summary>
    public enum FileStatus
    { 
        F ,     // just an ordinary file, not compressed
        C       // a compressed file.
    }

    /// <summary>
    /// Kelas ini merupakan kelas bantuan untuk melakukan kompresi
    /// data (stream) dalam format / algoritma GZIP, thanks to Microsoft :D
    /// </summary>
    class CompressHelper 
    {
        private static UInt32 origSize = 0;    // original file (byte[]) size.
        private static UInt32 compressedSize = 0 ;  // size ofter compressed.
        /// <summary>
        /// Original size of array of bytes would be compressed
        /// </summary>
        public static UInt32 OrigSize
        {
            get {
                return origSize;
            }
        }

        /// <summary>
        /// compressed size.
        /// </summary>
        public static UInt32 CompressedSize
        {
            get {
                return compressedSize;
            }
        }
        /// <summary>
        /// compress an array of bytes, and return the result
        /// </summary>
        /// <param name="bfile">array of bytes</param>
        /// <returns>an array of compressed bytes</returns>
	    public static byte[] compressByte(byte[] bfile)
	    {
            byte[] compressedbytes = null;
            origSize = (UInt32)bfile.Length;
            try
            {
                MemoryStream ms = new MemoryStream();
                // Use the newly created memory stream for the compressed data.
                GZipStream compressedzipStream = new GZipStream(ms, CompressionMode.Compress, true);
                // write the compressed bytes to buffer.
                compressedzipStream.Write(bfile, 0, bfile.Length);
                // Close the stream.
                compressedzipStream.Close();
                compressedSize = (uint) ms.Length;
                Console.WriteLine("Ukuran terkompresi = {0}",compressedSize);
                compressedbytes = new byte[ms.Length];
                int j = 0;
                ms.Position = 0;    // reset stream position
                while (j < ms.Length)
                {
                    compressedbytes[j++] = (byte)ms.ReadByte();
                }
            } // end try
            catch (InvalidDataException)
            {
                throw new Exception("Error: The file being read contains invalid data.");
            }
            catch (FileNotFoundException)
            {
                throw new Exception("Error:The file specified was not found.");
            }
            catch (ArgumentException)
            {
                throw new Exception("Error: path is a zero-length string, contains only white space, or contains one or more invalid characters");
            }
            catch (PathTooLongException)
            {
                throw new Exception("Error: The specified path, file name, or both exceed the system-defined maximum length. For example, on Windows-based platforms, paths must be less than 248 characters, and file names must be less than 260 characters.");
            }
            catch (DirectoryNotFoundException)
            {
                throw new Exception("Error: The specified path is invalid, such as being on an unmapped drive.");
            }
            catch (IOException)
            {
                throw new Exception("Error: An I/O error occurred while opening the file.");
            }
            catch (UnauthorizedAccessException)
            {
                throw new Exception("Error: path specified a file that is read-only, the path is a directory, or caller does not have the required permissions.");
            }
            catch (IndexOutOfRangeException)
            {
                throw new Exception("Error: You must provide parameters for MyGZIP.");
            }

            return compressedbytes;
	    }
    }
}

