using System;
using System.IO;

namespace Steganografi
{
	public class LSBHelper
	{
		
		// Create a Stream which hides the bytes contained 
		// in "message" by using the LSB method
        /// <summary>
        /// Melakukan streaming file dan menyimpan data bits informasi kedalamnya. 
        /// dengan menggunakan metode LSB
        /// file selanjutnya disimpan dalam stream outstream
        /// </summary>
        /// <param name="inStream">stream cover</param>
        /// <param name="message">larik byte pesan yang akan disimpan</param>
        /// <param name="outStream">stream file stego</param>
		public static void Encode(Stream inStream, byte[] message, Stream outStream) {
			
			int byteRead;
			byte byteWrite;
			int i = 0;
			int j = 0;
			byte bit;						
						
			while((byteRead = inStream.ReadByte()) != -1) 
			{																
				
				byteWrite = (byte) byteRead;
								
				if (i<message.Length) {
					
					// Extract the bit at the j position
					bit = Bit.Extract(message[i], j++);
					
					// Replace the LSB of byteWrite with "bit"
					Bit.Replace(ref byteWrite, 0, bit);	
				
					// Every 8 bits process another byte of "message"
					if (j==8) {j=0; i++;}					
				}
						
				outStream.WriteByte(byteWrite);
			}

            if (i < message.Length)
            {
                UInt32 len = (uint)message.Length;
                string msg = "Ukuran Cover terlalu kecil walaupun file yang disimpan dikompress,\n"+
                    " Hanya sebagian isi file yang disimpan\n ukuran file stelah dikompress adalah "+len + " bytes";
                throw new Exception();
            }
		}
		
        /// <summary>
        /// melakukan ekstraksi informasi yang akan disimpan.
        /// </summary>
        /// <param name="stream">stream data</param>
        /// <param name="length">panjang pesan</param>
        /// <returns></returns>
		public static byte[] Decode(Stream stream, int length) {
			
			byte[] hidden = new byte[length];
			int i = 0;
			int j = 0;
			byte bit;
			int byteRead;
			
			while((byteRead = stream.ReadByte()) != -1) 
			{												
				
				// Extract the LSB of byteRead
				bit = Bit.Extract((byte) byteRead, 0);
				
				// Replace the bit at the j position with "bit"
				Bit.Replace(ref hidden[i], j++, bit);
				
				// Every 8 bits process another byte of "hidden"
				if (j==8) {j=0; i++;}
				
				// Check bytes left
				if (i==length) break;												
			}		
			
			// The hidden var contains the hidden bytes
			return hidden;
			
		}		 
		
	}
	
}
