using System;
using System.Drawing;
using System.Windows.Forms;
using System.IO;

namespace Steganografi
{
    /// <summary>
    /// menyediakan antarmuka untuk operasi encoding dan decoding file
    /// dalam gambar (steganografi).
    ///     Author : Muhammad Hakim Asy'ari
    ///          Informatika ITB
    /// Note : sebenarnya program ini dibuat untuk melengkapi makalah yang saya tulis
    ///        sebagai salah satu tugas kuliah Kriptografi di IF di bawah bimbingan 
    ///        Bapak Rinaldi Munir S.T,M.T , Thanks Pak :)
    /// </summary>
    public partial class Steganografi : Form
    {
        private UInt32 dayatampung;     // estimation of dayatampung cover
        private UInt32 coverSize;
        private UInt32 infoSize;        // size of the file to be hide
        private FileStatus status;      // state = F | C
        private byte[] message = null;  // message yang akan disimpan.
        private UInt32 compressedSize = 0;

        public Steganografi()
        {
            InitializeComponent();
        }

        private void btnBrowseCover_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.Multiselect = false;
            ofd.Filter = "Windows Bitmap (*.bmp)|*.bmp| All Files | *.*";
            if (ofd.ShowDialog(this) != DialogResult.Cancel)
            {
                txbCover.Text = ofd.FileName;
                Image pic = new Bitmap(txbCover.Text);
                pcbCover.Image = new Bitmap(pic);
                pic.Dispose();
                FileInfo fi = new FileInfo(ofd.FileName);
                coverSize = (uint)fi.Length;
                lblSizeCover.Text = "Size: " + coverSize + " Bytes";
                dayatampung = (uint)((fi.Length - 115) / 8);
                lblDTCover.Text = "Daya Tampung: " + dayatampung.ToString() + " Bytes";
                //createBitmap(txbCover.Text);
            }
        }

        private void btnBrowseInfo_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.Multiselect = false;
            ofd.Filter = "All Files (*.*)|*.*";
            if (ofd.ShowDialog(this) != DialogResult.Cancel)
            {
                txbFileInfo.Text = ofd.FileName;
                FileInfo fi = new FileInfo(ofd.FileName);
                infoSize = (uint)fi.Length;
                lblSizeInfo.Text = "Size: " + infoSize + " Bytes";
                message = stream2Bytes(txbFileInfo.Text);
                CompressHelper.compressByte(message);
                compressedSize = CompressHelper.CompressedSize;
                lblCSize.Text = "Compressed size = " + compressedSize + " Bytes";
            }
        }

        private void btnSaveStego_Click(object sender, EventArgs e)
        {
            SaveFileDialog sfd = new SaveFileDialog();
            sfd.Filter = "Windows Bitmap (*.bmp)|*.bmp";
            if (sfd.ShowDialog(this) == DialogResult.OK)
            {
                txbFileStego.Text = sfd.FileName;
            }
        }

        private void btnEncode_Click(object sender, EventArgs e)
        {
            if (validasiInput())
            {
                this.Cursor = Cursors.WaitCursor;
                String stegoFile = txbFileStego.Text;
                if (!stegoFile.ToLower().EndsWith(".bmp"))
                    stegoFile += ".bmp";
                string pic = txbCover.Text;
                try
                {
                    long start = (long)DateTime.Now.Minute*60*1000+(long)DateTime.Now.Second*1000+(long)DateTime.Now.Millisecond;
                    // Open the cover
                    ICoverFile cover = new BMPCoverFile(createBitmap(pic,1));   // membuat objek.

                    string name = txbFileInfo.Text.Substring(txbFileInfo.Text.LastIndexOf("\\") + 1);

                    // periksa apakah file yang akan disimpan berukuran lebih
                    // besar dari ukuan daya tampung.
                    // jika lebih besar 
                    status = FileStatus.F;
                    if (infoSize > dayatampung) // maka
                    {
                        Console.WriteLine("Daya Tampung = {0} compressed = {1}",dayatampung,compressedSize);
                        //if (compressedSize > infoSize)
                        // periksa apakah hasil kompresi sudah cukup ditampung.
                        if (compressedSize > dayatampung)
                        {
                            uint needed = (uint)(compressedSize*8+115)/3;  // 115 = header,3 1 pixel menyimpan 3 byte.
                           // float factor = (float)needed / coverSize;
                            Console.WriteLine("Daya Tampung = {0} compressed = {1} faktor = {2}", dayatampung, compressedSize, needed);
                            cover = new BMPCoverFile(createBitmap(pic,needed));
                        }
                        status = FileStatus.C;
                    }
                    
                    // Create the stego file
                    cover.CreateStegoFile(stegoFile, message, name, txbKunci.Text.Trim(), status);
                    long end = (long)DateTime.Now.Minute * 60 * 1000 + (long)DateTime.Now.Second * 1000 + (long)DateTime.Now.Millisecond;

                    Console.WriteLine("elapsed time : {0}",end-start);
                    Result("File Telah Berhasil Disimpan");

                    Image stegoPic = new Bitmap(stegoFile);
                    pcbStego.Image = new Bitmap(stegoPic);
                    stegoPic.Dispose();

                }
                catch (Exception ex)
                {
                    Result(ex.Message);
                }
                finally
                {
                    //show normal cursor
                    this.Cursor = Cursors.Arrow;
                }
            }
        }

        private bool validasiInput()
        {
            // cover
            if (txbCover.Text == "")
            {
                MessageBox.Show("Masukkan alamat file cover");
                return false;
            }
            // file.
            if (txbFileInfo.Text == "")
            {
                MessageBox.Show("Masukkan alamat file yang akan disimpan");
                return false;
            }
            // stegofile.
            if (txbFileStego.Text == "")
            {
                MessageBox.Show("Masukkan alamat file stego akan disimpan");
                return false;
            }
            return true;
        }

        private void Result(string message)
        {

            MessageBox.Show(message);

        }

        private void btnBrowseStego_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.Multiselect = false;
            ofd.Filter = "Windows Bitmap (*.bmp)|*.bmp";
            if (ofd.ShowDialog(this) != DialogResult.Cancel)
            {
                txbFileStego2.Text = ofd.FileName;
                Image pic = new Bitmap(txbFileStego2.Text);
                pcbStego2.Image = new Bitmap(pic);
                pic.Dispose();
            }
        }

        private void btnDecode_Click(object sender, EventArgs e)
        {
            string stegoFile = txbFileStego2.Text;
            string password = txbKunci2.Text;

            if (stegoFile == "")
            {
                MessageBox.Show("Select the stego file to decode");
                return;
            }


            try
            {
                long start = (long)DateTime.Now.Minute * 60 * 1000 + (long)DateTime.Now.Second * 1000 + (long)DateTime.Now.Millisecond;
                // Open the stego file
                IStegoFile stego = new BMPStegoFile(stegoFile, password.Trim());

                // Show the hidden message
                string nama = "";
                byte[] message = stego.HiddenMessage;   // catch the hidden message.
                nama = stego.nama;  // catch the file name.
                int size = stego.Size;  // catch the file size.
                string msg = "File Terungkap \n Nama = " + nama + "\nUkuran = " + size + "\n Apakah Anda Ingin Menyimpannya ?";
                long end = (long)DateTime.Now.Minute * 60 * 1000 + (long)DateTime.Now.Second * 1000 + (long)DateTime.Now.Millisecond;
                Console.WriteLine("elapsed time decode : {0}",end- start);
                if (MessageBox.Show(msg, "Disimpan ?", MessageBoxButtons.YesNo,MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    SaveFileDialog sfd = new SaveFileDialog();
                    sfd.FileName = nama;
                    if (sfd.ShowDialog() == DialogResult.OK)
                    {
                        using (FileStream fs = new FileStream(sfd.FileName, FileMode.Create, FileAccess.Write))
                        {
                            fs.Write(message,0,message.Length);
                        }
                        MessageBox.Show("File Telah Disimpan dengan Nama "+nama);
                    }
                }
            }
            catch (Exception ex)
            {

                Result("Exception Euy!\n"+ex.Message);

            }
        }

        private byte[] stream2Bytes(string filename)
        {
            byte[] message = null;
            using (FileStream fs = new FileStream(filename, FileMode.Open, FileAccess.Read))
            {
                message = new byte[fs.Length];
                fs.Read(message, 0, message.Length);

                return message;
            }
        }

        /// <summary>
        /// Membuat clone file bitmap 24-bit from any image type (format)
        /// </summary>
        /// <param name="file">filename (path) to be cloned</param>
        /// <param name="size">new size</param>
        /// <returns>new 24-bit bitmap with new size</returns>
        private Bitmap createBitmap(string file,long size)
        {
            // distribute the factor for the height and the width
            Image img = Image.FromFile(file);            
            Size newSize = new Size(img.Width,img.Height);
            
            if (size > 1)
            {
                newSize = reSize(newSize, size);
            }
            Console.WriteLine("size = {0}",size);
            Console.WriteLine("new size = {0}",newSize);
            Bitmap bmp = new Bitmap(img,newSize);
            // clone Bitmap dengan format apaun menjadi 24-bit bitmap.
            System.Drawing.Rectangle rect = new System.Drawing.Rectangle(0, 0, bmp.Width, bmp.Height);
            bmp = bmp.Clone(rect, System.Drawing.Imaging.PixelFormat.Format24bppRgb);

            return bmp;
        }

        private Size reSize(Size s0, long max)
        {
            Size s = s0;
            long mul = s.Width * s.Height;
            int sum = s.Height + s.Height;

            while (mul < max)
            {
                s.Height += 1;
                s.Width += 1;
                mul = s.Width * s.Height;
                Console.WriteLine("mul = {0} < max = {1}",mul,max);
            }
            return s;
        }
        private void aboutMenu_Click(object sender, EventArgs e)
        {
            About ab = new About();
            ab.ShowDialog(this);
        }
    }
}